'''
Repo: https://github.com/Fitzy1293/handyderivatives
PyPi: https://pypi.org/project/handyderivatives/


Program to solve differentiable functions of one variable.
LHS defines what variable it's a function of.
No format checking right now.

Example of line separated file, one equation per line.

c(x) = r * (cos(x) + sqrt(-1)*sin(x))
a(t) = 1/2 * g * t ** 2
f(x) = sin(x**2) * x^2
h(w) = E ^ (w^4 - (3 * w)^2 + 9)
g(x) = exp(i * pi)
p(j) = csc(j^2)

With that it gives you the derivative, and both the equation and derivative in LaTeX.

'''

import sys
import os
import argparse
from argparse import RawTextHelpFormatter
from pprint import pprint

parser = argparse.ArgumentParser(
                                formatter_class=RawTextHelpFormatter,
                                description='Command line differential calculus tool using sympy.\nTry running:\nhandyderivatives -l -g \'f(x,y) = sin(x) * cos(y)\''
                                )
parser.add_argument(
                        '--input-file',
                		 '-f',
                		 dest='FILE',
                		 help='Input file'
                    )
parser.add_argument(
                        '--latex',
                        '-l',
                		dest='LATEX',
                		 action='store_true',
                		 help='Compile a LaTeX document as output'
         )
parser.add_argument(
                        '--diff',
                        '-d',
                        dest='DIFFERENTIAL',
                        nargs='*',
                        help='Works for equations written in the form  \'f(x) = x ^2\''
         )
parser.add_argument(
                        '--gradient',
                        '-g',
                        dest='GRADIENT',
                        nargs='*',
                        help='Works for scalar functions written in form  \'f(x,y,z) = x ^2 * sin(y) * cos(z)\''
         )


ARGS = parser.parse_args()

ARGC = len(sys.argv)
if ARGC == 1:
    parser.print_help()
    sys.exit()

if ARGS.FILE:
    if not os.path.exists(ARGS.FILE):
        parser.print_help()
        sys.exit(f"\n'{ARGS.FILE}' is not a file")

import subprocess # For running pdflatex
from sympy import sympify, diff, latex, Matrix
from sympy.abc import *


def makeLatexFile(equations):
    startStr    = '\\documentclass{article}\n\n\\usepackage{amsmath}\n\n\\begin{document}\n\n'
    endStr      = '\n\n\\end{document}'

    print('creating equations.tex')
    with open('equations.tex', 'w+') as f:
        f.write(startStr)
        for i, eq in enumerate(equations):
            if i % 2 == 0:
                f.write(f'({i // 2 + 1})\n\n' + '\\begin{align*}\n')
                f.write(f'\t{eq}\\\\')

            else:
                f.write(f'\t{eq}\n' +'\\end{align*}')
                f.write('\n')

            f.write('\n')


        f.write(f'{endStr}\n')

    print('compiling to equations.pdf')
    subprocess.run(['pdflatex', 'equations.tex', '-quiet'], stdout=subprocess.PIPE)
    for latexBuildFile in ('equations.log', 'equations.aux'):
        os.remove(latexBuildFile)


def cleanEquation(possibleEquation):
    if '#' in possibleEquation:
        possibleEquation = possibleEquation[:possibleEquation.find('#')]
    if possibleEquation.strip() in (''):
        return None
    else:
        return possibleEquation.replace('^', '**')

def sympyDifferentialData(equation):
    equationSplit            = equation.split('=')
    leftHand                 = equationSplit[0].strip()
    rightHand                = equationSplit[1].strip()

    differentiableExpression = sympify(rightHand)
    differentiableVariable   = sympify(leftHand[2])
    derivative               = diff(differentiableExpression, differentiableVariable)

    equationOutput           = f'{leftHand}{" " * 7}= {rightHand}'
    equationLatex            = f'{latex(sympify(leftHand))} &= {latex(differentiableExpression)}'

    derivativeEquationLeft   = f'd[{leftHand}]/d{differentiableVariable} = '
    derivativeOutput         = f'{derivativeEquationLeft}{derivative}'

    derivativeLeftHand       = '\\frac{d \\left[ ' + leftHand + ' \\right ] }' + '{d' + str(differentiableVariable) + '}'
    derivativeLatex          = f'{derivativeLeftHand} &= {latex(derivative)}'

    output = (
                f'\t{equationOutput}',
                f'\t{derivativeOutput}\n',
                f'\tTeX fnc.{" " * 5}{equationLatex}',
                f'\tTeX deriv.{" " * 3}{derivativeLatex}'
            )

    return ('\n'.join(output), (equationLatex, derivativeLatex))


def gradient(scalarFunctionStr):
    equation = cleanEquation(scalarFunctionStr)
    equationSplit                = equation.split('=')
    leftHand                     = equationSplit[0].strip()
    rightHand                    = equationSplit[1].strip()

    differentiableExpression     = sympify(rightHand)
    variables                    = leftHand.replace(' ', '')[2:-1].split(',')

    gradientComponents           = [diff(differentiableExpression, sympify(variable)) for variable in variables]

    latexLeftHand                = latex(sympify(leftHand))
    latexEquation                = f'{latexLeftHand} &= {latex(differentiableExpression)}'
    gradientVectorFormatting     = Matrix(gradientComponents)
    latexGradient                = f'\\nabla {latexLeftHand} &= {latex(gradientVectorFormatting)}'

    return (f'\t{equation}\n\t< {str(gradientComponents)[1:-1]} >', (latexEquation, latexGradient))


def printFmtDerivatives(file, strEquations=[]):
    equations = []

    if file:
        with open(file, 'r') as functionsFile:
            for equation in functionsFile.read().splitlines():
                equations.append((equation, 'diff-file'))

    if strEquations:
        for equation in strEquations:
            equations.append((equation, 'diff'))

    if ARGS.GRADIENT:
        for inputGradient in ARGS.GRADIENT:
            equations.append((inputGradient, 'gradient'))

    latexEquations  = []
    consoleOutput   = []
    for equationOperator in equations:
        equation = cleanEquation(equationOperator[0])
        operator = equationOperator[1]
        if not equation:
            continue

        if operator.startswith('diff'):
            terminalPrint, latexFunctionDerivativeTuple = sympyDifferentialData(equation)
        elif operator == 'gradient':
            terminalPrint, latexFunctionDerivativeTuple = gradient(equation)

        consoleOutput.append(terminalPrint)
        latexEquations.extend(latexFunctionDerivativeTuple)


    if ARGS.LATEX:
        makeLatexFile(latexEquations)
    else:
        for i, equationOutput in enumerate(consoleOutput):
            print(f'({i+1})\n{equationOutput}')


def main():
    printFmtDerivatives(ARGS.FILE, strEquations=ARGS.DIFFERENTIAL)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['handyderivatives']

package_data = \
{'': ['*']}

install_requires = \
['sympy>=1.7.1,<2.0.0']

entry_points = \
{'console_scripts': ['handyderivatives = handyderivatives:main']}

setup_kwargs = {
    'name': 'handyderivatives',
    'version': '2.1.2',
    'description': 'Calc. on the command line - with LaTeX output.',
    'long_description': "# handyderivatives\n\nThis is a command line program to get the derivatives for differentiable functions of a single variable.\n\n## Installation\n`pip install handyderivatives`\n\n[https://pypi.org/project/handyderivatives/](https://pypi.org/project/handyderivatives/)\n\n## Running it\nTo get the derivatives for an arbitrary number of functions of a single variable.\n\n`handyderivatives --latex -d 'f(x) = x ^ 2' 'g(x) = sin(x) + 2 * x'` ...\n\nTo get the gradient for an arbitrary number of scalar functions.\n\n`handyderivatives --latex -g 'f(x,y,z) = ln(x / (2 * y)) - z^2 * (x - 2 * y) - 3*z'` ...\n\nOr run that with one command.\n\n`handyderivatives -l -d 'f(x) = x ^ 2' 'g(x) = sin(x) + 2 * x' -g 'f(x,y,z) = ln(x / (2 * y)) - z^2 * (x - 2 * y) - 3*z'`\n\nTo differentiate a list of functions in a file.\n\n`handyderivatives --latex -f functions.txt`\n\n\n```\nusage: handyderivatives [-h] [--input-file FILE] [--latex] [--diff [DIFFERENTIAL [DIFFERENTIAL ...]]] [--gradient [GRADIENT [GRADIENT ...]]]\n\nCommand line differential calculus tool using sympy.\nTry running:\nhandyderivatives -l -g 'f(x,y) = sin(x) * cos(y)'\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --input-file FILE, -f FILE\n                        Input file\n  --latex, -l           Compile a LaTeX document as output\n  --diff [DIFFERENTIAL [DIFFERENTIAL ...]], -d [DIFFERENTIAL [DIFFERENTIAL ...]]\n                        Works for equations written in the form  'f(x) = x ^2'\n  --gradient [GRADIENT [GRADIENT ...]], -g [GRADIENT [GRADIENT ...]]\n                        Works for scalar functions written in form  'f(x,y,z) = x ^2 * sin(y) * cos(z)'\n```\n\n## Opening the output\nNormally you want to immediately see the output, so run something like this.\n\n`handyderivatives -l -d 'f(x) = sin(x)' && zathura equations.pdf --mode presentation`\n\nThe program used to open the PDF doesn't matter, as long as it's not something like Adobe Reader which takes a couple seconds to open on most machines.\nIf you can enter a PDF and it opens it, then it will work.\nZathura is nice because if you ctl + c in your terminal, or press q in the Zathura window, it will close the PDF.\nThis doesn't happen with all PDF viewers.\n\n## How the input file should be formatted\nEdit a file that has functions listed one per line.\nThe left hand side should be what your function will be differentiated with respect to, i.e *f(x)* .\nThe right hand side will be the expression.\n\n```\n# This is how the file for the argument -f should be formatted.\n\nc(x) = r * (cos(x) + sqrt(-1) * sin(x))\na(t) = 1/2 * g * t ** 2\nf(x) = sin(x**2) * x^2\nh(w) = E ^ (w^4 - (3 * w)^2 + 9)    # Capital E is interpreted by sympy as the base of the natural log.\ng(x) = exp(3 * pi)                  # So is exp(x), but written as a function taking an argument.\np(j) = csc(j^2)\n```\n\nIf you don't format it like that you will likely run into errors.\nYou  can add comments\n\n## TODO\nRecord screen while the program is executing for an example.\n\nAdd divergence.\n\n## LaTeX PDF output\n\n![Placeholder](https://raw.githubusercontent.com/Fitzy1293/handyderivatives/main/images/output.png)\n",
    'author': 'fitzy1293',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/fitzy1293/handyderivatives',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
